VERSION=1.2.2

SHELL=/bin/sh
CFLAGS=-Wall -ggdb -O2 
# -ffast-math option does not appear to produce a noticeable speedup.
export scriptdir=$(srcdir)/Scripts
export DESTDIR
export libdir
export POSSIBLESIZES
export CFLAGS

ALL_CFLAGS=-I. -I$(headerdir) -pthread $(CFLAGS)
MACRONAMES= NUMTHREADS BUFFERSIZE DEFAULTPARAMS DEFAULTVARIABLES DEFAULTMATRIX DEFAULTNUMTIMES DEFAULTTREE DEFAULTDATA DEFAULTMASKFILE DEFAULTMODELFILE ROOTDIR DEFAULTMARKUP DEFAULTOUTPUTTEMPLATE DEFAULTCODEFILE ONEFILE
VARIABLETAGS=  showeverysite path outputtemplate

DEFAULTVARIABLES=variables
POSSIBLESIZES=4 60 61 62

INSTALL=install
INSTALL_PROGRAM=$(INSTALL)
INSTALL_DATA=${INSTALL} -m 666
INSTALL_INFO=install-info

PACKAGENAME=cold
DNAPROGNAME=DNAml
UTILSETUPNAME=readmatricesfromcols
UTILSUMMATNAME=summats
UTILREORDERMATNAME=reordermatrix
UTILSTRIPAMBNAME=stripambiguous
UTILLOOKUPSPECIES=lookupspecies
UTILADDCODE=addcodebinary
UTILBOOTSTRAP=bootstrap
UTILGETPATRISTIC=getPatristic

ifdef IAMNOTROOT
prefix=$(HOME)
datarootdir=$(HOME)/share
infodir=$(HOME)/info
else
prefix=/usr/local
datarootdir=/usr/local/share
infodir=/usr/share/info
endif


exec_prefix=$(prefix)
bindir=$(exec_prefix)/bin
mainsearchdir=$(HOME)/$(PACKAGENAME)
docdir=$(datarootdir)/doc/$(PACKAGENAME)
libdir=$(exec_prefix)/lib/$(PACKAGENAME)
ROOTDIR=$(mainsearchdir)

ifdef KATHY
prefix=$(HOME)
libdir=$(HOME)/mirror2/lib/$(PACKAGENAME)
datarootdir=$(HOME)/mirror2
mainsearchdir=$(HOME)/mirror2/$(PACKAGENAME)
infodir=$(HOME)/mirror2/info
endif

export srcdir=$(shell pwd)
export headerdir=$(srcdir)/Headers

UTILS=$(UTILSETUPNAME) $(UTILREORDERMATNAME) $(UTILSUMMATNAME) $(UTILSTRIPAMBNAME) $(UTILLOOKUPSPECIES) $(UTILADDCODE) $(UTILBOOTSTRAP) $(UTILGETPATRISTIC)

UTILPATHS=$(foreach util,$(UTILS),Utilities/$(util))

installscripts=addcode addsize.sh
installscriptsfull=$(foreach script,$(installscripts),Scripts/$(script))

installbin=$(PACKAGENAME) $(UTILPATHS) $(installscriptsfull)


libs=Cold Distributions Optimisation Matrix TJK ColdOutput Output
inclibs=$(foreach lib,$(libs),-l$(lib))
installlib=$(foreach lib,$(libs),$(wildcard Libs/lib$(lib).so.*.*)) 
dynamiclib=$(wildcard Libs/libCode*.so.*.*)
installlib+=$(dynamiclib)

$(info $(installlib))

codes=universal
codedependantdata=parametermatrices ECMq.txt masks standardmodelmatrices gseriesmodelmats gtrmodelmats PCmodelmats
allcodedependantdata=$(codedependantdata) $(foreach code,$(codes),$(foreach data,$(codedependantdata),$(data)-$(code)))
modeldata=$(allcodedependantdata) mixture models varnames DNAmats AAmats PhysChemProperties kappamodelmats

exampledata=testtrees testdata 

utilitydata=codonlistexample.txt coldcodonorder.txt codonlistalph.txt

templates=DefaultOutputTemplate DefaultOutputTemplateHTML SummaryTemplate

abstemplates=$(foreach file,$(templates),Templates/$(file))
absmodeldata=$(foreach file,$(modeldata),Models/$(file))
absexampledata=$(foreach file,$(exampledata),Data/$(file))
absutilitydata=$(foreach file,$(utilitydata),Utilities/$(file))

installdat=$(DEFAULTVARIABLES) commandlinemacros environmentvars $(absmodeldata) $(absexampledata) $(absutilitydata) $(abstemplates) geneticcodes COPYING

installdoc=Documentation/cold.info

noninstalldoc=Documentation/old/cold.txt Documentation/old/files.txt Documentation/old/installation.txt Documentation/old/utilities.txt Documentation/cold.texi Documentation/LicenseInfo Documentation/fdl-1.3.texi

htmldocumentation=Documentation/cold.html Documentation/files.html Documentation/utilities.html Documentation/installation.html 

alldirs=$(DESTDIR)$(bindir) $(DESTDIR)$(datarootdir) $(DESTDIR)$(docdir) $(DESTDIR)$(infodir) $(DESTDIR)$(mainsearchdir) $(DESTDIR)$(mainsearchdir)/Data $(DESTDIR)$(mainsearchdir)/Models $(DESTDIR)$(mainsearchdir)/Utilities $(DESTDIR)$(mainsearchdir)/Templates $(DESTDIR)$(libdir)
#$(DESTDIR)$(datarootdir)/Models $(DESTDIR)$(datarootdir)/Data $(DESTDIR)$(datarootdir)/Utilities

ifdef VERSION
VERSIONTAIL=-$(VERSION)
endif

IDEF=ifdef
ELLSE=else
ENDDIF=endif

define NEWLINE


endef

forcereconfig:=
forceremakevar:=

define CHECKREMAKEVAR
$(IDEF) $(1)
forceremakevar:=FORCE
$(ENDDIF)
endef

INEQ=ifneq

define CHECKRECONFIG

$(IDEF) $(1)

$(INEQ) ($$(shell awk '{if ($$$$2=="$(1)") print $$$$3}' $$(headerdir)/configvars.h),$$($(1)))

$$(warning forcing reconfiguration)

forcereconfig:=FORCE

$(ENDDIF)
$(ENDDIF)
endef


DistributionsLibCPPFiles=Distributions

MatrixLibCPPFiles=Matrix MatrixIO

TJKLibCPPFiles=CommandLine ErrorHandler filesearch Miscellaneous SignalHandler

OptimisationLibCPPFiles=Optimisation

ColdLibCPPFiles=Debug Derivatives DNASequence DNAroutines Hessian Hessian2 Input MixTreeLikelihood Mixture Parameters Sequence Simulation Tree TreeLikelihood Initialise Parsimony OutputFunctionassign Placeholders preprocessorloop innerloop innerloop1 innerloop2 innerloop3 innerloop4 

OutputLibCPPFiles=Computations Index Markup Output Placeholders

define listlibfiles
$1LibFiles=$(foreach cppfile,$($1LibCPPFiles),$(cppfile).cpp) makefile MadeLib

endef

export Libraries=Distributions TJK Matrix Optimisation Cold Output

$(foreach lib,$(Libraries),$(eval $(call listlibfiles,$(lib))))


AllLibfiles=$(foreach lib,$(Libraries),$(foreach dlf,$($(lib)LibFiles),$(lib)/$(dlf))) makefile MadeLibs

Outputcppfiles=Output

Maincppfiles=Arithmetic Debug Initialisation Input Main Testing

Utilitycppfiles=reordermatrix summats stripambiguous lookupspecies setup addcode bootstrap getPatristic

Javafiles=FrontEnd/ColdFrontend.java FrontEnd/cold-frontend.jar FrontEnd/manifest FrontEnd/makefile


LibFiles=$(foreach lib,$(AllLibfiles),Libs/$(lib))

OutputFiles=$(foreach out,$(Outputcppfiles),Output/$(out).cpp)

MainFiles=$(foreach main,$(Maincppfiles),$(main).cpp)

UtilityFiles=$(foreach utility,$(Utilitycppfiles),Utilities/$(utility).cpp)

coldcppfiles=$(LibFiles) $(OutputFiles) $(MainFiles) 

allcppfiles=$(coldcppfiles) $(UtilityFiles) 

alldocfiles=$(htmldocumentation) $(installdoc) $(noninstalldoc)

autoheads=configvars.h flags.h markupcommands.h modifiers.h placeholders.h markupcommands2.h modifiers2.h placeholders2.h placeholders3.h validcommands.h recovercommands.h recovercommandsunderride.h 

allheads=CommandLine.h Input.h Likelihood.h Matrix.h Miscellaneous.h MiscellaneousFuns.h Optimisation.h Parameters.h Sequence.h SignalHandler.h Tree.h tree_private.h TreeLikelihood.h TreeLikelihood_base.h MixTreeLikelihood.h ErrorHandler.h Macros.h ModelFields.h fileExtensions.h DNAMacros.h Output.h Testing.h Distributions.h State.h Computations.h Index.h $(autoheads) PlaceholderTemplate.cpp

automaticheaders=$(foreach head,$(autoheads),Headers/$(head))

allheaders=$(foreach head,$(allheads),Headers/$(head))

scripts=mkdirp makeheaders.sh makemacrodefs makefunctionnamemacros autogenmessage addcode.template 

fullscripts=$(foreach scr,$(scripts),Scripts/$(scr)) $(installscriptsfull)

compileddata=.flags .markupcommands .modifiers .placeholders configvars.template .validcommands .recovercommands .recovercommandsunderride

compileddat=$(foreach dat,$(compileddata),Headers/$(dat))

alldata=$(compileddat) $(installdat)

alldistfiles=$(allcppfiles) $(allheaders) makefile targets.mk mkfilevariables.mk .variables.template $(fullscripts) $(alldocfiles) $(alldata) $(Javafiles)


tarfilename=$(PACKAGENAME)$(VERSIONTAIL).tar

projectdir=Hong/programs/Cold
backupdir=$(HOME)/$(projectdir)/Backups
timestamp:=$(shell date +%s)
backupname=$(PACKAGENAME)_backup$(VERSIONTAIL).$(timestamp)

SVVARS=PACKAGENAME UTILSETUPNAME UTILSUMMATNAME UTILREORDERMATNAME UTILSTRIPAMBNAME UTILLOOKUPSPECIES UTILADDCODE UTILBOOTSTRAP mainsearchdir DEFAULTVARIABLES prefix datarootdir infodir libdir VERSION DESTDIR POSSIBLESIZES


SAVEVARS=$(shell rm LastMake) $(foreach svVAR,$(SVVARS), $(shell echo $(svVAR)=$($(svVAR)) >> LastMake))


ifeq ($(MAKECMDGOALS),)

$(SAVEVARS)

else ifeq ($(MAKECMDGOALS),all)

$(SAVEVARS)

else ifeq ($(MAKECMDGOALS),$(PACKAGENAME))

$(SAVEVARS)

else ifeq ($(MAKECMDGOALS),install)

ifeq ($(shell test -f LastMake && echo y),y)
include LastMake
endif

else ifeq ($(MAKECMDGOALS),uninstall)

$(info Uninstalling cold)
ifeq ($(shell test -f LastMake && echo y),y)
include LastMake
endif

else ifeq ($(MAKECMDGOALS),update)

ifeq ($(shell test -f LastMake && echo y),y)
include LastMake
endif

endif 

installedfiles=

$(foreach libfile,$(installlib), $(eval installedfiles += $(DESTDIR)$(libdir)/$(notdir $(libfile))))
$(foreach libfile,$(installlib), $(eval installedfiles += $(DESTDIR)$(libdir)/$(basename $(notdir $(libfile)))))
$(foreach libfile,$(installlib), $(eval installedfiles += $(DESTDIR)$(libdir)/$(basename $(basename $(notdir $(libfile))))))
$(foreach libfile,$(installlib), $(eval installedfiles += $(DESTDIR)$(libdir)/$(basename $(basename $(basename $(notdir $(libfile)))))))
$(foreach libfile,$(installlib), $(eval installedfiles += $(DESTDIR)$(libdir)/$(basename $(basename $(basename $(basename $(notdir $(libfile))))))))
$(foreach progfile,$(installbin), $(eval installedfiles += $(DESTDIR)$(bindir)/$(notdir $(progfile))))
$(foreach datfile,$(installdat), $(eval installedfiles += $(DESTDIR)$(mainsearchdir)/$(datfile)))
$(foreach docfile,$(installdoc), $(eval installedfiles += $(DESTDIR)$(infodir)/$(docfile)))
$(foreach docfile,$(htmldocumentation), $(eval installedfiles += $(DESTDIR)$(docdir)/html/$(progfile)))

